"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginManagerService = exports.PluginContainer = void 0;
const log_service_1 = require("../services/log-service");
const constants_1 = require("../models/constants");
const session_type_1 = require("../models/session-type");
const operating_system_1 = require("../models/operating-system");
const aws_credentials_plugin_1 = require("./aws-credentials-plugin");
class PluginContainer {
    constructor(metadata) {
        this.metadata = metadata;
        this.pluginInstances = [];
    }
}
exports.PluginContainer = PluginContainer;
class PluginManagerService {
    constructor(pluginEnvironment, nativeService, logService, repository, sessionFactory, http) {
        this.pluginEnvironment = pluginEnvironment;
        this.nativeService = nativeService;
        this.logService = logService;
        this.repository = repository;
        this.sessionFactory = sessionFactory;
        this.http = http;
        this._pluginDir = "plugins";
        this._pluginContainers = [];
        this._requireModule = nativeService.requireModule;
        this._hashElement = nativeService.hashElement.hashElement;
    }
    static base64Decoding(input) {
        return Buffer.from(input, "base64");
    }
    get pluginContainers() {
        return this._pluginContainers;
    }
    getPluginByName(name) {
        return this._pluginContainers.find((plugin) => plugin.metadata.uniqueName === name);
    }
    verifyAndGeneratePluginFolderIfMissing() {
        if (!this.nativeService.fs.existsSync(this.nativeService.os.homedir() + "/.Leapp/" + this._pluginDir)) {
            this.nativeService.fs.mkdirSync(this.nativeService.os.homedir() + "/.Leapp/" + this._pluginDir);
        }
    }
    async loadFromPluginDir() {
        this._pluginContainers = [];
        const options = {
            folders: { include: ["*.*"] },
            files: { exclude: ["signature", ".DS_Store", "package-lock.json"] },
        };
        const pluginDirContent = this.nativeService.fs.readdirSync(this.nativeService.os.homedir() + "/.Leapp/" + this._pluginDir);
        for (const pluginName of pluginDirContent) {
            const pluginFilePath = this.nativeService.os.homedir() + "/.Leapp/" + this._pluginDir + "/" + pluginName;
            const isDir = this.nativeService.fs.existsSync(pluginFilePath) && this.nativeService.fs.lstatSync(pluginFilePath).isDirectory();
            if (isDir) {
                // VALIDATION PROCESS
                const { packageJson, isPluginValid } = await this.validatePlugin(pluginFilePath, options, pluginName);
                // HANDLE PACKAGE.JSON ERROR
                let metadata;
                try {
                    metadata = this.extractMetadata(packageJson);
                }
                catch (errors) {
                    this.logService.log(new log_service_1.LoggedEntry(`missing or invalid values in plugin ${pluginName} package.json: ${errors.message}`, this, log_service_1.LogLevel.warn, true));
                    continue;
                }
                // CHECK VALIDATION
                if (!isPluginValid) {
                    this.logService.log(new log_service_1.LoggedEntry(`Signature not verified for plugin: ${pluginName}`, this, log_service_1.LogLevel.warn, true));
                    await this.nativeService.fs.remove(pluginFilePath);
                    continue;
                }
                // LOAD
                try {
                    if (this.nativeService.fs.existsSync(pluginFilePath + "/plugin.js")) {
                        const pluginModule = this._requireModule(pluginFilePath + "/plugin.js");
                        this.logService.log(new log_service_1.LoggedEntry(`loading ${pluginName} plugin`, this, log_service_1.LogLevel.info, false));
                        const pluginContainer = new PluginContainer(metadata);
                        const pluginClasses = Object.values(pluginModule);
                        for (const pluginClass of pluginClasses) {
                            const pluginInstance = new pluginClass(this.pluginEnvironment, this.sessionFactory);
                            pluginInstance.metadata = metadata;
                            pluginContainer.pluginInstances.push(pluginInstance);
                        }
                        this._pluginContainers.push(pluginContainer);
                        if (!this.repository.getPluginStatus(metadata.uniqueName)) {
                            this.repository.createPluginStatus(metadata.uniqueName);
                        }
                    }
                }
                catch (error) {
                    this.logService.log(new log_service_1.LoggedException(`error loading plugin ${pluginName}: ${error.message}`, this, log_service_1.LogLevel.error, true));
                }
            }
        }
    }
    unloadAllPlugins() {
        this._pluginContainers = [];
    }
    unloadSinglePlugin(name) {
        const pluginIndex = this._pluginContainers.map((p) => p.metadata.uniqueName).indexOf(name);
        if (pluginIndex > -1) {
            this._pluginContainers.splice(pluginIndex, 1);
        }
    }
    testRsaSignToBase64(message) {
        const privateKey = "fake-rsa-key";
        const signer = this.nativeService.crypto.createSign("sha256");
        signer.update(message);
        signer.end();
        const signature = signer.sign({
            key: privateKey,
            format: "pem",
            type: "pkcs1",
            passphrase: "fake-passphrase",
        });
        return signature.toString("Base64");
    }
    availableAwsCredentialsPlugins(os, session) {
        const list = this._pluginContainers.filter((plugin) => {
            var _a, _b, _c, _d, _e, _f;
            const active = (_b = this.repository.getPluginStatus((_a = plugin === null || plugin === void 0 ? void 0 : plugin.metadata) === null || _a === void 0 ? void 0 : _a.uniqueName)) === null || _b === void 0 ? void 0 : _b.active;
            const supportedOS = (_d = (_c = plugin === null || plugin === void 0 ? void 0 : plugin.metadata) === null || _c === void 0 ? void 0 : _c.supportedOS) === null || _d === void 0 ? void 0 : _d.includes(os);
            const supportedSome = (_f = (_e = plugin === null || plugin === void 0 ? void 0 : plugin.metadata) === null || _e === void 0 ? void 0 : _e.supportedSessions) === null || _f === void 0 ? void 0 : _f.some((supportedSession) => this.sessionFactory.getCompatibleTypes(supportedSession).includes(session.type));
            return active && supportedOS && supportedSome;
        });
        let arrayToReturn = [];
        list.forEach((pluginContainer) => {
            arrayToReturn = arrayToReturn.concat(pluginContainer.pluginInstances.filter((plugin) => plugin.pluginType === aws_credentials_plugin_1.AwsCredentialsPlugin.name));
        });
        return arrayToReturn;
    }
    async installPlugin(url) {
        const packageName = url.replace("leapp://", "");
        const pluginsDir = this.nativeService.os.homedir() + "/.Leapp/plugins";
        this.logService.log(new log_service_1.LoggedEntry(`We are ready to install Plugin ${packageName}, please wait...`, this, log_service_1.LogLevel.info, true));
        const npmMetadata = await this.http.get(`https://registry.npmjs.org/${packageName}`, { responseType: "json" }).toPromise();
        if (!npmMetadata["keywords"] || !npmMetadata["keywords"].includes("leapp-plugin")) {
            throw new log_service_1.LoggedException(`${npmMetadata["name"]} is not a Leapp plugin`, this, log_service_1.LogLevel.error, true);
        }
        const version = npmMetadata["dist-tags"].latest;
        const tarballUrl = npmMetadata.versions[version].dist.tarball;
        const tarballPathComponents = tarballUrl.split("/");
        const tarballFileName = tarballPathComponents[tarballPathComponents.length - 1];
        const tarballBuffer = await this.http.get(tarballUrl, { responseType: "arraybuffer" }).toPromise();
        const tarballFilePath = this.nativeService.path.join(pluginsDir, tarballFileName);
        this.nativeService.fs.writeFileSync(tarballFilePath, Buffer.from(tarballBuffer));
        const pluginDir = this.nativeService.path.join(pluginsDir, packageName);
        await this.nativeService.fs.remove(pluginDir);
        await this.nativeService.fs.ensureDir(pluginDir);
        await this.nativeService.tar.x({
            file: tarballFilePath,
            strip: 1,
            ["C"]: pluginDir,
        });
        await this.nativeService.fs.remove(tarballFilePath);
        this.logService.log(new log_service_1.LoggedEntry(`Plugin ${packageName} installed correctly.`, this, log_service_1.LogLevel.info, true));
    }
    extractMetadata(packageJson) {
        var _a;
        const errors = [];
        const version = packageJson.version;
        if (!version) {
            errors.push("version");
        }
        const uniqueName = packageJson.name;
        if (!uniqueName) {
            errors.push("name");
        }
        const author = ((_a = packageJson.author) === null || _a === void 0 ? void 0 : _a.name) ? packageJson.author.name : packageJson.author;
        if (!author) {
            errors.push("author");
        }
        const description = packageJson.description;
        if (!description) {
            errors.push("description");
        }
        const keywords = packageJson.keywords;
        if (!keywords || keywords.length === 0) {
            errors.push("keywords");
        }
        else if (!keywords.includes(constants_1.constants.npmRequiredPluginKeyword)) {
            errors.push(`${constants_1.constants.npmRequiredPluginKeyword} keyword`);
        }
        const leappPluginConfig = packageJson.leappPlugin;
        if (!leappPluginConfig) {
            errors.push("leappPlugin");
        }
        const supportedSessionTypes = (leappPluginConfig === null || leappPluginConfig === void 0 ? void 0 : leappPluginConfig.supportedSessions) || [session_type_1.SessionType.anytype];
        for (const sessionType of supportedSessionTypes) {
            if (this.sessionFactory.getCompatibleTypes(sessionType).length === 0) {
                errors.push(`leappPlugin.supportedSessions: ${sessionType} is unsupported`);
            }
        }
        const icon = (leappPluginConfig === null || leappPluginConfig === void 0 ? void 0 : leappPluginConfig.icon) || "fas fa-puzzle-piece";
        const operatingSystems = [operating_system_1.OperatingSystem.mac, operating_system_1.OperatingSystem.linux, operating_system_1.OperatingSystem.windows];
        const supportedOS = (leappPluginConfig === null || leappPluginConfig === void 0 ? void 0 : leappPluginConfig.supportedOS) || operatingSystems;
        for (const os of supportedOS) {
            if (!operatingSystems.includes(os)) {
                errors.push(`leappPlugin.supportedOS: ${os} is unsupported`);
            }
        }
        const url = leappPluginConfig === null || leappPluginConfig === void 0 ? void 0 : leappPluginConfig.url;
        if (errors.length) {
            throw new Error(errors.join(", "));
        }
        const pluginStatus = this.repository.getPluginStatus(uniqueName);
        return {
            version,
            active: pluginStatus ? pluginStatus.active : true,
            author,
            description,
            supportedOS,
            supportedSessions: supportedSessionTypes,
            icon,
            keywords,
            uniqueName,
            url,
        };
    }
    async validatePlugin(pluginFilePath, options, pluginName) {
        let packageJson;
        try {
            // Hashing file and directory
            const hash = await this._hashElement(pluginFilePath, options);
            if (!hash.children) {
                return { packageJson, isPluginValid: false };
            }
            // If it has children then it is a directory
            if (
            // Required files
            this.nativeService.fs.existsSync(pluginFilePath + "/package.json") &&
                this.nativeService.fs.existsSync(pluginFilePath + "/plugin.js")) {
                const packageJsonContent = this.nativeService.fs.readFileSync(pluginFilePath + "/package.json");
                packageJson = JSON.parse(packageJsonContent);
                if (this.skipPluginValidation()) {
                    return { packageJson, isPluginValid: true };
                }
                // Verify signature to enable plugin
                const data = await this.http.get(constants_1.constants.pluginPortalUrl + `/${pluginName}`, { responseType: "json" }).toPromise();
                if (data.status !== "active") {
                    this.logService.log(new log_service_1.LoggedEntry("Plugin not in active state: " + pluginName, this, log_service_1.LogLevel.warn, true));
                    return { packageJson, isPluginValid: false };
                }
                const verifyMessage = packageJsonContent + hash.hash;
                const signatureVerified = this.rsaVerifySignatureFromBase64(constants_1.constants.publicKey, verifyMessage, data.signature);
                if (!signatureVerified) {
                    return { packageJson, isPluginValid: false };
                }
            }
            else {
                this.logService.log(new log_service_1.LoggedEntry(`folder ${pluginFilePath} is not a plugin folder, ignoring...`, this, log_service_1.LogLevel.info, false));
                return { packageJson, isPluginValid: false };
            }
        }
        catch (error) {
            this.logService.log(new log_service_1.LoggedException(`hashing failed or verification failed: ${error.message}`, this, log_service_1.LogLevel.warn, false));
            return { packageJson, isPluginValid: false };
        }
        return { packageJson, isPluginValid: true };
    }
    rsaVerifySignatureFromBase64(publicKey, message, signatureBase64) {
        const signature = PluginManagerService.base64Decoding(signatureBase64);
        const verifier = this.nativeService.crypto.createVerify("sha256");
        verifier.update(message);
        verifier.end();
        return verifier.verify(publicKey, signature);
    }
    skipPluginValidation() {
        return constants_1.constants.skipPluginValidation;
    }
}
exports.PluginManagerService = PluginManagerService;
//# sourceMappingURL=plugin-manager-service.js.map